# In alphabetical order

import re

# Extract list patterns from regex
def extract_pattern(text):
    pattern = r"\$[^\s]+|\[\[\d+\]\]"
    return re.findall(pattern, text)


# Extract elements with a given pattern
def extract_elements_with_pattern(lst):
    pattern = r"\$([^\s]+)|\[\[(\d+)\]\]"
    extracted_elements = []
    for item in lst:
        match = re.match(pattern, item)
        if match:
            extracted_elements.append(match.group(1) or match.group(2))
    return extracted_elements


# Flatten a list of lists
def flatten_list(x):
    return [val for sublist in x for val in sublist]


# Formatting object as a string
def format_value(value):
    return f"'{value}'"


# check is string contains an R vector
def is_vector(x):
    return ("[1]" in x) and (x != "")


# check is string contains an R matrix
def is_matrix(x):
    return ("[1,]" in x) and (x != "")


def remove_elements_with_pattern(lst):
    pattern = r"\$[^\s]+|\[\[\d+\]\]"
    return [item for item in lst[1:] if not re.match(pattern, str(item))]


# split string based on a pattern
def split_string(text):
    pattern = r"(\$[^\s]+|\[\[\d+\]\])"
    return re.split(pattern, text)


# obtain matrix from string
def str_to_matrix(x):

    # Split the 'y' part into lines
    y_part_lines = x.strip().split("\n")

    # Extract column names
    column_names = y_part_lines[0].split()

    # Initialize empty lists to store numeric column values
    numeric_columns = []

    # Find the minimum length among all rows
    min_length = min(len(line.split()) for line in y_part_lines)

    # Iterate over each line
    for line in y_part_lines:
        # Split each line based on whitespace
        values = line.split(maxsplit=1)[1].split()
        # Keep only the last min_length elements
        values = values[-min_length:]
        # Check if the elements contain numeric values
        numeric_values = [
            float(val) for val in values if re.match(r"^-?\d+\.?\d*$", val)
        ]
        # Include column names as the first element of the first list
        if not numeric_columns:
            numeric_values = [column_names] + numeric_values
        numeric_columns.append(numeric_values)

    if len(numeric_columns[1:]) == 1:
        return numeric_columns[1:][0]
    return numeric_columns[1:]


# obtain vector from string
def str_to_vector(x):

    # Split the 'y' part into lines
    y_part_lines = x.strip().split("\n")

    # Initialize an empty list to store the parsed values
    parsed_y_values = []

    # Iterate over each line and extract numeric values
    for line in y_part_lines:
        # Split each line based on whitespace
        values = line.split()
        # Extract numeric values and convert them to float
        numeric_values = [float(val) for val in values[1:]]
        # Append the numeric values to the parsed list
        parsed_y_values.append(numeric_values)

    res = flatten_list(parsed_y_values)
    if len(res) == 1:
        return res[0]
    return res



"""Main module."""

import re
import subprocess
from functools import lru_cache
from .utils import *


@lru_cache  # size is 128 'results'
def callfunc(r_code, r_func="my_func", type_return="float", **kwargs):
    """

    `callfunc` calls an R function `r_func` defined in an R code `r_code`.

    # Parameters:

    `r_code` (str): R code to be executed for the function `r_func` to work. Must
    use double braces around function definition, and a semi-colon (';') after
    each instruction.

    `r_func` (str): name of the R function (defined in `r_code`) being called.

    `type_return` (str): 	type of function return. Either "int", "float", "list" and "dict".

    `kwargs`: not used (yet)

    # Example:

    ```python
    import rtopy as rp

    # an R function that returns the product of an arbitrary number of arguments
    # notice the double braces around the R function's code
    # and the a semi-colon (';') after each instruction
    r_code1 = 'my_func <- function(arg1=NULL, arg2=NULL, arg3=NULL, arg4=NULL, arg5=NULL)
                {{
                    args <- c(arg1, arg2, arg3, arg4, arg5);
                    args <- args[!sapply(args, is.null)];
                    result <- prod(args);
                    return(result)
                }}'

    print(rp.callfunc(r_code=r_code1, r_func="my_func", type_return="int",
    arg1=3, arg2=5, arg3=2))
    ```

    See also [https://github.com/Techtonique/rtopy/blob/main/rtopy/demo/thierrymoudiki_20240304_rtopyintro.ipynb](https://github.com/Techtonique/rtopy/blob/main/rtopy/demo/thierrymoudiki_20240304_rtopyintro.ipynb)

    """

    r_code_ = r_code + ";"

    # Constructing argument string for the R function call
    arg_string = ""
    for key, value in kwargs.items():
        if isinstance(value, (float, int, list)):
            arg_string += f"{key}={value}, "
        elif isinstance(value, str):
            arg_string += f"{key}={format_value(value)}, "
    arg_string = arg_string[:-2]  # remove last comma and trailing space
    r_code_ += f"{r_func}({arg_string})"
    r_code_ = r_code_.replace(" ", "").replace("\n", "")
    result = subprocess.run(
        ["Rscript", "-e", r_code_], capture_output=True, text=True, check=True
    ).stdout

    if type_return in ("int", "float", "list"):
        result = result.split("\n")[-2].strip().replace("[1] ", "")

    if type_return == "dict":
        keys = extract_elements_with_pattern(extract_pattern(result))
        # Initialize an empty dictionary to store the lists
        result_dict = {}
        # Iterate over sections and extract key-value pairs
        r_list_sections = split_string(result)
        r_list_values = remove_elements_with_pattern(r_list_sections)
        for idx, key in enumerate(keys):
            try:
                section = r_list_values[idx]
                if is_vector(section):
                    result_dict[keys[idx]] = str_to_vector(section)
                elif is_matrix(section):
                    result_dict[keys[idx]] = str_to_matrix(section)
                else:
                    continue
            except:
                continue

    if type_return == "float":
        return float(result)

    elif type_return == "int":
        return int(result)

    elif type_return == "list":
        return [float(elt) for elt in result.split(" ")]

    elif type_return == "dict":
        return result_dict



#!/usr/bin/env python

"""The setup script."""

import platform
import subprocess

def check_r_installed():
    current_platform = platform.system()

    if current_platform == "Windows":
        # Check if R is installed on Windows by checking the registry
        try:
            subprocess.run(
                ["reg", "query", "HKLM\\Software\\R-core\\R"], check=True
            )
            print("R is already installed on Windows.")
            return True
        except subprocess.CalledProcessError:
            print("R is not installed on Windows.")
            return False

    elif current_platform == "Linux":
        # Check if R is installed on Linux by checking if the 'R' executable is available
        try:
            subprocess.run(["which", "R"], check=True)
            print("R is already installed on Linux.")
            return True
        except subprocess.CalledProcessError:
            print("R is not installed on Linux.")
            return False

    elif current_platform == "Darwin":  # macOS
        # Check if R is installed on macOS by checking if the 'R' executable is available
        try:
            subprocess.run(["which", "R"], check=True)
            print("R is already installed on macOS.")
            return True
        except subprocess.CalledProcessError:
            print("R is not installed on macOS.")
            return False

    else:
        print("Unsupported platform. Unable to check for R installation.")
        return False

def install_r():

    current_platform = platform.system()

    if current_platform == "Windows":
        # Install R on Windows using PowerShell
        install_command = "Start-Process powershell -Verb subprocess.runAs -ArgumentList '-Command \"& {Invoke-WebRequest https://cran.r-project.org/bin/windows/base/R-4.1.2-win.exe -OutFile R.exe}; Start-Process R.exe -ArgumentList '/SILENT' -Wait}'"
        subprocess.run(install_command, shell=True)

    elif current_platform == "Linux":
        # Install R on Linux using the appropriate package manager (e.g., apt-get)
        install_command = (
            "sudo apt update -qq && sudo apt-key adv --keyserver keyserver.ubuntu.com --recv-keys E298A3A825C0D65DFD57CBB651716619E084DAB9"
            + "&& sudo add-apt-repository 'deb https://cloud.r-project.org/bin/linux/ubuntu focal-cran40/'"
            + "&& sudo apt update"
            + "&& sudo apt -y install r-base"
        )
        subprocess.run(install_command, shell=True)

    elif current_platform == "Darwin":  # macOS
        # Install R on macOS using Homebrew
        install_command = "brew install r"
        subprocess.run(install_command, shell=True)

    else:

        print("Unsupported platform. Unable to install R.")

def install_packages():
    try:    
        subprocess.run(["Rscript", "--default-packages=methods, datasets, utils, grDevices, graphics, stats", "-e", "utils::install.packages('Rcpp', repos='https://cran.rstudio.com', dependencies=TRUE)"])         
    except:     
        try:
            subprocess.run(["mkdir", "-p", "r-pkgs"])
            subprocess.run(["Rscript", "--default-packages=methods, datasets, utils, grDevices, graphics, stats", "-e", "utils::install.packages('Rcpp', repos='https://cran.rstudio.com', lib='r-pkgs', dependencies=TRUE)"])            
        except:             
            try:
                subprocess.run(["sudo", "Rscript", "--default-packages=methods, datasets, utils, grDevices, graphics, stats", "-e", "utils::install.packages('Rcpp', repos='https://cran.rstudio.com', dependencies=TRUE)"])
            except:
                subprocess.run(["mkdir", "-p", "r-pkgs"])
                subprocess.run(["sudo", "Rscript", "--default-packages=methods, datasets, utils, grDevices, graphics, stats", "-e", "utils::install.packages('Rcpp', repos='https://cran.rstudio.com', lib='r-pkgs', dependencies=TRUE)"])


# Check if R is installed; if not, install it
if not check_r_installed():
    print("Installing R...")
    install_r()
else:
    print("No installation needed.")

install_packages()

subprocess.run(["pip", "install", "rpy2"])

from setuptools import setup, find_packages
from codecs import open
from os import path

__version__ = "0.1.1"

here = path.abspath(path.dirname(__file__))

# get the dependencies and installs

with open(
    path.join(here, "requirements.txt"), encoding="utf-8"
) as f:
    all_reqs = f.read().split("\n")

install_requires = [
    x.strip() for x in all_reqs if "git+" not in x
]
dependency_links = [
    x.strip().replace("git+", "")
    for x in all_reqs
    if x.startswith("git+")
]

setup(
    author="T. Moudiki",
    author_email='thierry.moudiki@gmail.com',
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="Call R functions from Python",
    entry_points={
        'console_scripts': [
            'rtopy=rtopy.cli:main',
        ],
    },
    install_requires=install_requires,
    license="BSD license",
    long_description="Call R functions from Python (using command line utilities and text mining)",
    include_package_data=True,
    keywords='rtopy',
    name='rtopy',
    packages=find_packages(include=['rtopy', 'rtopy.*']),
    test_suite='tests',
    url='https://github.com/thierrymoudiki/rtopy',
    version='0.1.1',
    zip_safe=False,
)
